local aUF = LibStub("AceAddon-3.0"):GetAddon("ag_UnitFrames")
local newFrame, delFrame = aUF.newFrame, aUF.delFrame

local plugin = aUF:NewModule("Healthbar")
-- Healthbar can't be disabled
plugin.hideFromMenu = true

local barColor = {}
local activeframes = {}
local DURATION = 0.5
local fadeFrame = CreateFrame("Frame")
local _abs, _cos, _pi = math.abs, math.cos, math.pi

local function CosineInterpolate(y1, y2, mu)
	local mu2 = (1-_cos(mu*_pi))/2
	return y1*(1-mu2)+y2*mu2
end

local function updateFaders(self)
	local now, stop, alpha = GetTime()
	local ran = false
	for frame, object in pairs(activeframes) do
		ran = true
		stop, alpha = frame.stop
		if stop < now then
			-- done.  set both bar values to the dest value, nuke entry in activeframes, set alpha to 0
			activeframes[frame] = nil
			-- set skipFade == true
			plugin.UpdateHealth(object, true)
			frame.startValue = nil
			frame.destValue = nil
			frame:Hide()
			return
		end
		if frame.startValue < frame.destValue then
			alpha = (1 - ((stop - now) / DURATION))*0.7
		else
			alpha = ((stop - now) / DURATION)*0.7
		end
		if frame.style == "flash" then
			frame:SetAlpha(alpha)
		elseif frame.style == 'both' then
			frame:SetAlpha(alpha)
			local cVal = frame:GetValue()
			cVal = CosineInterpolate(cVal,frame.destValue, 1 - ((stop - now) / DURATION) )
			frame:SetValue(cVal)
		elseif frame.style == 'smooth' then
			local cVal = frame:GetValue()
			cVal = CosineInterpolate(cVal,frame.destValue, 1 - ((stop - now) / DURATION) )
			frame:SetValue(cVal)
		end
	end
	if not ran then
		self:Hide()
	end
end

function plugin:OnRegister()
	self:RegisterDefaults({
		units = {
			["**"] = {
				ClassColorBars = true,
			},
			["target"] = {
				TargetShowHostile = true,
			}
		}
	})

	self.subConfig = "Bars"
end

function plugin:OnEnable()
	aUF:RegisterBarType("HealthBar")
end

function plugin:OnObjectEnable(object)
	local bars = object.bars
	if not bars.HealthBar then
		bars.HealthBar = newFrame("StatusBar", object.frame)	
		bars.HealthBar.bg = newFrame("Texture", object.frame, "BORDER")
		bars.HealthBar:SetScript("OnValueChanged",function() self.StatusBarsOnValueChanged(object, arg1) end)
		bars.HealthBar:SetMinMaxValues(0,100)
	end
end

function plugin:OnRegisterEvents(object)
	object:RegisterUnitEvent("UNIT_HEALTH", self.UpdateHealth)
	object:RegisterUnitEvent("UNIT_MAXHEALTH", self.UpdateHealth)
	object:RegisterUnitEvent("UNIT_THREAT_SITUATION_UPDATE", self.UpdateAggro)
	if object.inherit == "pet" then
		object:ObjectRegisterEvent("UNIT_HAPPINESS", self.HealthBarColor)
	end
end

function plugin:OnObjectDisable(object)
	object:UnregisterUnitEvent("UNIT_HEALTH", self.UpdateHealth)
	object:UnregisterUnitEvent("UNIT_MAXHEALTH", self.UpdateHealth)
	object:UnregisterUnitEvent("UNIT_THREAT_SITUATION_UPDATE", self.UpdateAggro)
	if object.inherit == "pet" then
		object:ObjectUnregisterEvent("UNIT_HAPPINESS", self.HealthBarColor)
	end
end

function plugin:OnUpdateAll(object)
	self.UpdateHealth(object)
	self.HealthBarColor(object)
end

function plugin:OnMetroUpdate(object)
	self.UpdateHealth(object)
end

function plugin:OnStatusBarsColor(object)
    if object.bars.HealthBar then
        self.HealthBarColor(object)
    end
end

function plugin:OnLayoutApplied(object)
	if object.bars.HealthBar then
		object.bars.HealthBar.bg:ClearAllPoints()
		object.bars.HealthBar.bg:SetAllPoints(object.bars.HealthBar)
	end
end

function plugin:OnSetBarTexture(object)
	local texture = aUF:GetBarTexture()
	if object.bars.HealthBar then
		object.bars.HealthBar:SetStatusBarTexture(texture,"BORDER")
	end
	if object.bars.HealthBar.bg then
		object.bars.HealthBar.bg:SetTexture(texture,"BORDER")
	end
	if object.healthFade then
		object.healthFade:SetStatusBarTexture(texture)
	end
end

local targets, victims, aggro, oldaggro = {}, {}, {}, {}
function plugin.UpdateAggro(object)
	local status = UnitThreatSituation(object.unit)
	if plugin.db.profile.units[object.type].AggroHealth and status and status == 3 then
		object.aggro = true
	else
		object.aggro = nil
	end
end

local function DrawFader(object)
	local healthFade
	if object.healthFade then
		healthFade = object.healthFade
	else
		healthFade = newFrame("StatusBar", object.frame)
	end
	object.healthFade = healthFade
	healthFade.object = object
	healthFade.destValue = nil
	healthFade.faderstop = nil
	healthFade:SetOrientation(object.bars.HealthBar:GetOrientation())
	healthFade:SetAllPoints(object.bars.HealthBar)
	healthFade:SetMinMaxValues(object.bars.HealthBar:GetMinMaxValues())
	healthFade:SetValue(object.bars.HealthBar:GetValue())
	healthFade:SetStatusBarTexture(aUF:GetBarTexture())
	healthFade:Hide()
	-- hack in setting the updater function
	fadeFrame:SetScript("OnUpdate", updateFaders)
	return healthFade
end

---------------
-- STATUSBAR --
---------------

function plugin.UpdateHealth(self, skipFade)
	local currValue,maxValue = UnitHealth(self.unit), UnitHealthMax(self.unit)
	local perc = currValue/maxValue * 100

	if ( not UnitExists(self.unit) or UnitIsDead(self.unit) or UnitIsGhost(self.unit) or not UnitIsConnected(self.unit) or maxValue == 1) then
		perc = 0
		self.bars.HealthBar:SetValue(0)
	end
	if plugin.db.profile.units[self.type].HealthDeficit then
		perc = 100 - perc
	end
	local db = plugin.db.profile.units[self.type]
	if skipFade ~= true and (db.SmoothHealth or db.FadeHealth) then
		DURATION = aUF.db.profile.BarFadeTime
		if not self.healthFade then DrawFader(self) end
		local preValue = self.healthFade.destValue or self.bars.HealthBar:GetValue()
		local destValue = (maxValue == 0 and 0) or perc
		if _abs(destValue - preValue) >= 0.015 then
			local healthFade = self.healthFade
			if (not healthFade.startValue) then
				healthFade.startValue = preValue
				isFading = false
			end
			healthFade.stop = GetTime() + DURATION
			healthFade.destValue = destValue
			if (db.SmoothHealth and db.FadeHealth) then
				if preValue < destValue then
					self.bars.HealthBar:SetValue(preValue)
				end
				if (not isFading) then
					healthFade:SetValue(preValue)
				end
				healthFade:SetAlpha(0)
				healthFade.style = 'both'
			elseif db.SmoothHealth then
				if preValue < destValue then
					self.bars.HealthBar:SetValue(preValue)
				end
				if (not isFading) then
					healthFade:SetValue(preValue)
				end
				healthFade:SetAlpha(0.7)
				healthFade.style = "smooth"
			elseif db.FadeHealth then
				if preValue < destValue then
					healthFade:SetAlpha(0)
					self.bars.HealthBar:SetValue(preValue)
					if (not isFading) then
						healthFade:SetValue(destValue)
					end
				else
					if (not isFading) then
						healthFade:SetAlpha(0.7)
						healthFade:SetValue(preValue)
					end
				end
				healthFade.style = "flash"
			end
			healthFade:Show()
			activeframes[healthFade] = self
			fadeFrame:Show()
		end
	end
	self.bars.HealthBar:SetValue(perc)
end

---------------------
-- STATUSBAR COLOR --
---------------------

local color1 = {r=0.88,g=0.17,b=0.29}
local color2 = {r=1,g=0.97,b=0.1}
function plugin.HealthBarColor(self)
	local db = aUF.db.profile
	local healthColor
	local _,eClass = UnitClass(self.unit)

	if self.unit:find("target") and not UnitIsFriend(self.unit, "player") and (plugin.db.profile.units[self.type].TargetShowHostile == true ) and UnitExists(self.unit) then
		healthColor = aUF:UtilFactionColors(self.unit)
		if aUF.db.profile.SmoothHealthBars == true then
			barColor[self] = 2
		else
			barColor[self] = nil
		end
	elseif self.unit == "pet" and eClass ~= "WARLOCK" then
		local happiness = GetPetHappiness()
		if ( happiness == 1 ) then
			healthColor = color1
		elseif ( happiness == 2 ) then
			healthColor = color2
		else
			healthColor = db.HealthColor
		end
		barColor[self] = nil
	elseif UnitIsPlayer(self.unit) and plugin.db.profile.units[self.type].ClassColorBars == true and RAID_CLASS_COLORS[eClass] then
		healthColor = RAID_CLASS_COLORS[eClass]
		barColor[self] = nil
	else
		healthColor = db.HealthColor
		barColor[self] = 1
	end

	if ( barColor[self] ) then
		plugin.StatusBarsOnValueChanged(self, self.bars.HealthBar:GetValue())
	else
		if plugin.db.profile.units[self.type].AggroHealth and self.aggro then
			self.bars.HealthBar:SetStatusBarColor(1, 0, 0, 1)
		else
			self.bars.HealthBar:SetStatusBarColor(healthColor.r,healthColor.g,healthColor.b, 1)
		end
	end

	self.bars.HealthBar.bg:SetVertexColor(healthColor.r,healthColor.g,healthColor.b,0.25)
	if (plugin.db.profile.units[self.type].SmoothHealth or plugin.db.profile.units[self.type].FadeHealth) then
		self.bars.HealthBar:SetAlpha(1)
	else
		self.bars.HealthBar:SetAlpha(.8)
	end

	if self.healthFade then
		self.healthFade:SetStatusBarColor(self.bars.HealthBar:GetStatusBarColor())
	end
end

function plugin.StatusBarsOnValueChanged(self, value)
	if barColor[self] == 1 then
		plugin.SmoothHealthBarOnValueChanged(self, value)
	elseif barColor[self] == 2 then
		plugin.SmoothTargetHealthBarOnValueChanged(self, value, aUF:UtilFactionColors(self.unit))
	end
end

function plugin.SmoothTargetHealthBarOnValueChanged(self, value, colortable)
	if ( not value) then
		return
	end
	local r, g, b
	local min, max = self.bars.HealthBar:GetMinMaxValues()
	if ( (value < min) or (value > max) ) then
		return
	end
	if ( (max - min) > 0 ) then
		value = (value - min) / (max - min)
	else
		value = 0
	end

	r = colortable.r*(0.35*value+0.65)
	g = colortable.g*(0.35*value+0.65)
	b = colortable.b*(0.35*value+0.65)
	if plugin.db.profile.units[self.type].AggroHealth and self.aggro then
		r = 1
		g = 0
		b = 0
	end
	self.bars.HealthBar:SetStatusBarColor(r, g, b)
	if self.healthFade then
		self.healthFade:SetStatusBarColor(r, g, b)
	end
end

function plugin.SmoothHealthBarOnValueChanged(self, value)
	if ( not value ) then
		return
	end
	local db = aUF.db.profile
	local r, g, b
	local min, max = self.bars.HealthBar:GetMinMaxValues()
	if ( (value < min) or (value > max) ) then
		return
	end
	if ( (max - min) > 0 ) then
		value = (value - min) / (max - min)
	else
		value = 0
	end
	if(value > 0.5) then
		r = (db.HealthColor.r) + (((1-value) * 2)* (1-(db.HealthColor.r)))
		g = db.HealthColor.g
	else
		r = 1.0
		g = (db.HealthColor.g) - (0.5 - value)* 2 * (db.HealthColor.g)
	end
	b = db.HealthColor.b
	if plugin.db.profile.units[self.type].AggroHealth and self.aggro then
		r = 1
		g = 0
		b = 0
	end
	self.bars.HealthBar:SetStatusBarColor(r, g, b)
	if self.healthFade then
		self.healthFade:SetStatusBarColor(r, g, b)
	end
end
